@@ -2,7 +2,7 @@
  *
  * This is the layer separating protocol processing from actual io.
  *
- * $Id: abstract_io.h,v 1.34 2001/03/14 15:12:14 nisse Exp $ */
+ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -64,11 +64,15 @@
 		; make sure that handler is not called again. '
 
 		; Available is 0 iff we have got EOF.
-		UINT32 "UINT32 available" "UINT8 *data")))
+		uint32_t "uint32_t available" "const uint8_t *data")))
 		"struct exception_handler *e")))
 */
 
 /* May store a new handler into *h. */
 #define READ_HANDLER(h, l, d) ((h)->handler(&(h), (l), (d)))
 
+struct read_handler *
+make_read_file(struct abstract_write *c,
+	       uint32_t max_size);
+
 #endif /*LSH_ABSTRACT_IO_H_INCLUDED */
