@@ -26,9 +26,9 @@
 #include "lsh.h"
 
 /* Can we avoid this dependency? */
-#include "nettle/bignum.h"
-#include "nettle/base64.h"
-#include "nettle/nettle-meta.h"
+#include <nettle/bignum.h>
+#include <nettle/base64.h>
+#include <nettle/nettle-meta.h>
 
 /* The memory allocation model (for strings) is as follows:
  *
@@ -54,7 +54,7 @@ struct lsh_string *
 lsh_string_alloc_clue(uint32_t size, const char *clue);
 
 #define lsh_string_alloc(size) \
-  (lsh_string_alloc_clue((size), (__FILE__ ":" STRING_LINE ": " FUNCTION_NAME)))
+  (lsh_string_alloc_clue((size), (__FILE__ ":" STRING_LINE)))
 
 void
 lsh_string_final_check(void);
@@ -150,6 +150,13 @@ lsh_string_cbc_decrypt(struct lsh_string *dst, uint32_t di,
 		       uint32_t block_size, uint8_t *iv,
 		       nettle_crypt_func f, void *ctx);
 
+void
+lsh_string_ctr_crypt(struct lsh_string *dst, uint32_t di,
+		     const struct lsh_string *src, uint32_t si,
+		     uint32_t length,
+		     uint32_t block_size, uint8_t *iv,
+		     nettle_crypt_func f, void *ctx);
+
 void
 lsh_string_write_hash(struct lsh_string *s, uint32_t start,
 		      const struct nettle_hash *type, void *ctx);
