@@ -28,8 +28,8 @@
 #include <assert.h>
 #include <string.h>
 
-#include "nettle/bignum.h"
-#include "nettle/macros.h"
+#include <nettle/bignum.h>
+#include <nettle/macros.h>
 
 #include "parse.h"
 
@@ -129,65 +129,120 @@ parse_uint8(struct simple_buffer *buffer, unsigned *result)
 }
 
 int
-parse_utf8(struct simple_buffer *buffer, uint32_t *result)
+parse_utf8(struct simple_buffer *buffer, uint32_t *result, unsigned *utf8_length)
 {
-  uint32_t first;
-  int length;
-  int i;
+  static const uint32_t min_value[7] =
+    {
+      0, 0,
+           0x80,
+          0x800,
+        0x10000,
+       0x200000,
+      0x4000000,
+    };
+
+  uint32_t c;
+  uint32_t value;
+  unsigned length;
+  unsigned i;
   
   if (!LEFT)
     return -1;
 
-  first = HERE[0];
+  c = HERE[0];
 
-  if (first < 0x80)
+  if (c < 0x80)
     {
-      *result = first;
+      *result = c;
+      *utf8_length = 1;
       ADVANCE(1);
       return 1;
     }
 
-  switch(first & 0xF0)
+  switch(c & 0xF0)
     {
     default:
+      ADVANCE(1);
+      *utf8_length = 1;
       return 0;
     case 0xC0:
     case 0xD0:
+      /* Format 110y yyyy 10xx xxxx, 11 bits */
       length = 2;
-      *result = first & 0x1F;
+      value = c & 0x1F;      
       break;
     case 0xE0:
+      /* Format 1110 zzzz 10yy yyyy 10xx xxxx, 16 bits */
       length = 3;
-      *result = first & 0x0F;
+      value = c & 0x0F;
       break;
     case 0xF0:
-      switch(first & 0x0E)
+      switch(c & 0x0E)
 	{
 	case 0: case 2: case 4: case 6:
+	  /* Format 1111 0www 10zz zzzz 10yy yyyy 10xx xxxx, 21 bits */
 	  length = 4;
-	  *result = first & 0x07;
+	  value = c & 0x07;
 	  break;
 	case 8: case 0xA:
+	  /* Format 1111 10xx 10ww wwww 10zz zzzz 10yy yyyy 10xx xxxx, 26 bits */
 	  length = 5;
-	  *result = first & 0x03;
+	  value = c & 0x03;
 	  break;
 	case 0xC:
+	  /* Format 1111 110y 10xx xxxx 10ww wwww 10zz zzzz 10yy yyyy 10xx xxxx, 31 bits */
 	  length = 6;
-	  *result = first & 0x01;
+	  value = c & 0x01;
 	  break;
 	default:
-	  fatal("Internal error!\n");
+	  /* Invalid format 1111 111x */
+	  ADVANCE(1);
+	  return 0;
 	}
       break;
     }
-  for(i = 1; i<length; i++)
+  if (LEFT < length)
     {
-      uint32_t c = HERE[i];
-      if ( (c & 0xC0) != 0x80)
-	return 0;
-      *result = (*result << 6) | (c & 0x3f);
+      ADVANCE(LEFT);
+      *utf8_length = 1;
+      return 0;
     }
+  c = HERE[1];
+  
+  if ( (c & 0xC0) != 0x80)
+    {
+      ADVANCE(1);
+      *utf8_length = 1;
+      return 0;  
+    }
+  value = (value << 6) | (c & 0x3f);
+
+  for(i = 2; i<length; i++)
+    {
+      c = HERE[i];
+      if ( (c & 0xC0) != 0x80)
+	{
+	  ADVANCE(i);
+	  *utf8_length = i;
+	  return 0;
+	}
+      value = (value << 6) | (c & 0x3f);
+    }
+
   ADVANCE(length);
+  *utf8_length = length;
+  
+  /* Check for overlong sequences */
+  if (value < min_value[length])
+    return 0;
+
+  /* Surrogates and non-characters should not appear in utf8 text. */
+  if ( (value >= 0xd800 && value <0xe000)
+       || value == 0xfffe || value == 0xffff)
+    return 0;
+  
+  *result = value;
+  
   return 1;
 }  
       
