@@ -1,6 +1,5 @@
 /* rsync.h
  *
- * $Id: rsync.h,v 1.7 2001/06/17 19:30:28 nisse Exp $
  */
 
 /* 
@@ -26,16 +25,6 @@
 #ifndef RSYNC_H_INCLUDED
 #define RSYNC_H_INCLUDED
 
-#if LSH
-# include "lsh_types.h"
-#else
-# if HAVE_CONFIG_H
-#  include "config.h"
-# endif
-#endif
-
-#include <inttypes.h>
-
 #include <nettle/md5.h>
 
 #include <stdlib.h>
