@@ -30,8 +30,8 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "nettle/buffer.h"
-#include "nettle/sexp.h"
+#include <nettle/buffer.h>
+#include <nettle/sexp.h>
 
 #include "tag.h"
 
