@@ -145,7 +145,8 @@ make_pwhelper_callback(struct unix_user *user,
   return &self->super;
 }
 
-/* NOTE: Consumes the pw string if successful. */
+/* NOTE: Consumes the pw string if successful.
+   FIXME: Should be renamed. */
 static int
 kerberos_check_pw(struct unix_user *user, struct lsh_string *pw,
 		  struct command_continuation *c,
@@ -221,7 +222,7 @@ kerberos_check_pw(struct unix_user *user, struct lsh_string *pw,
 
 	close(in[0]);
 
-	fd = io_write(make_lsh_fd(in[1], "password helper stdin",
+	fd = io_write(make_lsh_fd(in[1], IO_NORMAL, "password helper stdin",
 				  e),
 		      lsh_string_length(pw), NULL);
 
@@ -501,7 +502,7 @@ do_read_file(struct lsh_user *u,
       /* NOTE: We could install an exit handler for the child process,
        * but there's nothing useful for that to do. */
       io_read(make_lsh_fd
-	        (out[0], "stdout, reading a user file",
+	        (out[0], IO_NORMAL, "stdout, reading a user file",
 		 make_exc_read_user_file_handler(c,
 						 &default_exception_handler,
 						 HANDLER_CONTEXT)),
@@ -1134,15 +1135,19 @@ do_lookup_user(struct user_db *s,
       /* Check for root login */
       if (!passwd->pw_uid && !self->allow_root)
 	goto fail;
-      
-#if HAVE_GETSPNAM
+
+#if HAVE_GETSPNAM && HAVE_SHADOW_H
+      {
+      struct spwd *shadowpwd;
+
       /* FIXME: What's the most portable way to test for shadow
        * passwords? For now, we look up shadow database if and only if
-       * the passwd field equals "x". */
-      if (!strcmp(crypted, "x"))
+       * the passwd field equals "x". If there's no shadow record, we
+       * just keep the value from the passwd-database, the user may be
+       * able to login using a publickey, or the password helper. */
+      if (strcmp(crypted, "x") == 0
+	  && (shadowpwd = getspnam(cname)))
 	{
-	  struct spwd *shadowpwd;
-
 	  /* Current day number since January 1, 1970.
 	   *
 	   * FIXME: Which timezone is used in the /etc/shadow file? */
@@ -1190,6 +1195,7 @@ do_lookup_user(struct user_db *s,
 
 	  crypted = shadowpwd->sp_pwdp;
 	}
+      }
 #endif /* HAVE_GETSPNAM */
       /* Check again for empty passwd field (as it may have been
        * replaced by the shadow one). */
